#include "uhttps.h"
#include "cfg_parser.h"
#include <sys/stat.h>


static boolean _fexist(char* filename) // check if a file exist. ALTERNATIVES: http://stackoverflow.com/questions/230062/whats-the-best-way-to-check-if-a-file-exists-in-c-cross-platform
{
	struct stat buf;
	int i = stat ( filename, &buf );
	if ( i == 0 ) return true; // file found
	else return false;
}

static boolean _isdir(char* pathname) // ritorna "true" se pathname è una directory, o false altrimenti
{
	struct stat buf;
	stat( pathname, &buf );
	if( S_ISDIR( buf.st_mode ) ) return true;
	else return false;
}


void uhttps_configuration_print( const struct uhttp_configuration* current_config )
{
	printf("%s=%u\n", "use_ipv6", current_config->use_ipv6 );
	printf("%s=%u\n", "listening_port", current_config->listening_port );
	printf("%s=%s\n", "root_pathname", current_config->root_pathname );
	printf("%s=%s\n", "log_pathname", current_config->log_pathname );
	printf("%s=%s\n", "usersdb_pathname", current_config->usersdb_pathname );
	printf("%s=%u\n", "max_threads", current_config->max_threads );
	printf("%s=%u\n", "max_processes", current_config->max_processes );
	printf("%s=%u\n", "max_connected_clients_count", current_config->max_connected_clients_count );
	printf("%s=%u\n", "max_cached_file_size", current_config->max_cached_file_size );
	printf("%s=%u\n", "max_cached_file_count", current_config->max_cached_file_count );
}

void uhttps_configuration_init( struct uhttp_configuration* current_config )
{
	memset( current_config, 0, sizeof(current_config) );
	
	// initialize to defaults
	current_config->use_ipv6 = true;
	current_config->listening_port = 80;
	strcpy( current_config->root_pathname, "." );
	strcpy( current_config->log_pathname, "access.log" );
	strcpy( current_config->usersdb_pathname, "uhttps_usersdb.txt" );
	current_config->max_threads = 10;
	current_config->max_processes = 1;
	current_config->max_connected_clients_count = 10; // max( current_config->max_threads, current_config->max_processes )
	current_config->max_cached_file_size = 1024;
	current_config->max_cached_file_count = 10;
}

void uhttps_configuration_read( struct uhttp_configuration* current_config, const String cfgfile_path )
{
	// TODO? bloccare il file durante la lettura
	
	// apre il file di configurazione in sola lettura
	FILE* cfgfile = fopen(cfgfile_path, "r");
	if( !cfgfile )
	{
		uhttps_log_error( "[warn] no config file \"%s\" found, keeping defaults", cfgfile_path );
		return;
	}
	
	// legge e parsa una riga alla volta
	char current_line[ _UHTTPS_MAX_CFG_FILE_LINE_LENGHT ] = {0};
	char current_field[ _UHTTPS_MAX_CFG_FILE_LINE_LENGHT ] = {0};
	char current_value[ _UHTTPS_MAX_CFG_FILE_LINE_LENGHT ] = {0};
	int current_line_count = 0;
	while( fgets( current_line, _UHTTPS_MAX_CFG_FILE_LINE_LENGHT, cfgfile ) != NULL )
	{
		current_line_count++;
		
		// se la linea è vuota o inizia con "#" (commento), la ignora
		if(*current_line=='\r' || *current_line=='\n' || *current_line=='#') continue;
		// altrimenti prova il parsing
		*current_field='\0'; *current_value='\0';
		sscanf( current_line, "%[^=]=%s", current_field, current_value );
		if(!*current_field || !*current_value)
		{
			uhttps_log_error( "[error] invalid line %d in config file, skipping", current_line_count );
			continue;
		}
		
		// identifica il campo e lo scrive in current_config
		if( strcmp( current_field, "use_ipv6") == 0 )
		{
			if( strcmp(current_value, "yes") == 0 ) current_config->use_ipv6=true;
			if( strcmp(current_value, "1") == 0 ) current_config->use_ipv6=true;
			if( strcmp(current_value, "no") == 0 ) current_config->use_ipv6=false;
			if( strcmp(current_value, "0") == 0 ) current_config->use_ipv6=false;
		}
		if( strcmp( current_field, "listening_port") == 0 )
		{
			current_config->listening_port = atoi(current_value);
			if( current_config->listening_port == 0 ) // TODO? limitare range
			{
				uhttps_log_error( "[error] invalid port number (%u), using default (80)", current_config->listening_port);
				current_config->listening_port = 80;
			}
		}
		else if( strcmp( current_field, "log_pathname") == 0 )
		{
			strcpy( current_config->log_pathname, current_value );
			// OSS.: il file può anche non esistere...
		}
		else if( strcmp( current_field, "root_pathname") == 0 )
		{
			if( _isdir( current_value ) )
			{
				strcpy( current_config->root_pathname, current_value );
			}
			else
			{
				uhttps_log_error( "[error] root directory \"%s\" does not exist, using default (cwd)" );
				strcpy( current_config->root_pathname, "." );
			}
			
		}
		else if( strcmp( current_field, "usersdb_pathname") == 0 )
		{
			strcpy( current_config->usersdb_pathname, current_value );
			// test esistenza effettuato nel modulo "usersdb.c"...
		}
		else if( strcmp( current_field, "max_threads") == 0 )
		{
			current_config->max_threads = atoi(current_value);
			// può essere anche 0...
		}
		else if( strcmp( current_field, "max_processes") == 0 )
		{
			current_config->max_processes = atoi(current_value);
			// può essere anche 0...
		}
// NO? max_connected_clients_count -> ricavato dai precedenti
		else if( strcmp( current_field, "max_cached_file_size") == 0 )
		{
			current_config->max_cached_file_size = atoi(current_value);
			// può essere anche 0...
		}
		else if( strcmp( current_field, "max_cached_file_count") == 0 )
		{
			current_config->max_cached_file_count = atoi(current_value);
			// se è <=0 la cache è disattivata...
		}
		//else if( strcmp( current_field, "max_cached_file_time") == 0 )
		//	current_config->max_cached_file_time = atoi(current_value);
		else
		{
			uhttps_log_error( "[error] invalid line %d in config file, skipping", current_line_count );
			continue;
		}
		
	} // END while
		
	// chiude il file di configurazione
	fclose( cfgfile );
	
	// stampa alcuni avvisi a seconda della configurazione attuale
		if( !uhttps_file_cache_is_enabled(current_config) )
			uhttps_log_error( "[notice] file cache disabled" );
		
		if( uhttps_is_multithreaded(current_config) )
		{
			uhttps_log_error( "[notice] server is set in multithreaded mode.");
		}
		else
		{
			#if defined(WINDOWS) || defined(WIN32) || defined(_WIN32)
				uhttps_log_error("[warn] Windows version does not support multiple process, using threads instead.");
				current_config->max_processes = 1;
			#else
				uhttps_log_error( "[notice] server is set in multiprocess mode.");
			#endif
		}
}


boolean uhttps_is_multithreaded( const struct uhttp_configuration* current_config )
{
	if(( current_config->max_threads > 1 ) &&
	   ( (current_config->max_threads) >= (current_config->max_processes) )
	  )
		return true;
	else
		return false;
}

boolean uhttps_file_cache_is_enabled( const struct uhttp_configuration* current_config )
{
	return( (current_config->max_cached_file_count >0 ) && (current_config->max_cached_file_size > 0 ) );
}
