#include "uhttps.h"
#include "http_parser.h"
#include <string.h>
#include <strings.h> // for strcasecmp
#include "base64.h"
#include <ctype.h> // for isxdigit


static void _decode_url(char* url)
{
	// decodifica i chars escaped nella url (sintassi: "%hh")
	// la stringa è modificata "in place"
	// http://en.wikipedia.org/wiki/Percent-encoding
	// http://bytes.com/topic/c/answers/212081-function-url-decode-string
	// 2FIX: accented chars special encoding
	
// DEBUG: puts(url);
	
	char* src = url;
	char* dst = url;
	char code[3] = {0};
	unsigned long ascii = 0;
	char *end = NULL;

	while(*src)
	{
		if( *src == '%' &&
			*(src+1) &&
		    *(src+2) &&
		    isxdigit(*(src+1)) && 
		    isxdigit(*(src+2))
		  ) {
			src++;
			strncpy(code, src, 2);
			ascii = strtoul(code, &end, 16); // converte il codice esadecimale
			if(ascii <= 0)
			{
				// se la conversione è fallita mantiene il char invariato
				*dst++ = *src++;
				continue;
			}
			*dst = (char)ascii;
			dst++;
			src += 2;
// DEBUG: printf("\t%s=%c \n", code, ascii);
		}
		else
		{
			*dst++ = *src++; // copia il char corrente così com'è
		}
	}
	*dst = '\0';
	
// DEBUG: puts(url);
}


void uhttps_parse_http_request_headers( struct uttps_http_request_parsed* out_parsed_request, char* in_request )
{
	if( !in_request || !out_parsed_request ) return;
	
// DEBUG: puts("request:");
// DEBUG: puts(in_request);
	
	// parse the Initial Request Line
	int r = sscanf( in_request, "%10s %500s HTTP/%5s", out_parsed_request->method, out_parsed_request->url, out_parsed_request->http_version);
	if( r == EOF ) return;
	
	// decodifica i char escaped nella url (sintassi: "%hh")
	_decode_url( out_parsed_request->url );
	
	// skip to the next line
	while(*in_request && *in_request!='\n') in_request++;
	 in_request++;
	
	// parse Header Lines
	char header_line_field[_UHTTPS_MAX_HTTP_HEADER_LINE_LENGTH] = {0};
	char header_line_values[_UHTTPS_MAX_HTTP_HEADER_LINE_LENGTH] = {0};
	while( sscanf( in_request, "%500[^ :]:%*[ \t]%500[^\r]", header_line_field, header_line_values) == 2 ) // 2FIX: NON legge gli headers multilinea
	{
// DEBUG: printf("\tparsed header: %s=%s\n", header_line_field, header_line_values);
		
		// riconosce l'header e lo copia nella struct out_parsed_request
		if( strcasecmp(header_line_field, "From") == 0 )
			strcpy(out_parsed_request->from, header_line_values);
		else if( strcasecmp(header_line_field, "User-Agent") == 0 )
			strcpy(out_parsed_request->user_agent, header_line_values);
		else if( strcasecmp(header_line_field, "Authorization") == 0 )
		{
			// decodifica username e password in base64
			char base64_encoded_login[_UHTTPS_MAX_HTTP_HEADER_LINE_LENGTH] = {0};
			strcpy( base64_encoded_login, header_line_values+6 ); // ignora la stringa "Basic "
			b64decode( base64_encoded_login );  // decodifica il login
			char* login_separator = strchr(base64_encoded_login, ':'); // trova il separatore tra username e password
			// if(login_separator==NULL) ?
			*login_separator = '\0'; // spezza il login in due stringhe sovrascrivendo il separatore
			strcpy(out_parsed_request->username, base64_encoded_login); // copia l'username (la prima parte del login)
			strcpy(out_parsed_request->password, login_separator+1); // copia la password (la parte del login dopo il separatore)
		}
		else if( strcasecmp(header_line_field, "Content-Type") == 0 )
			strcpy(out_parsed_request->content_type, header_line_values);
		else if( strcasecmp(header_line_field, "Content-Length") == 0 )
			out_parsed_request->content_length = (size_t) atol(header_line_values);
		//else 
		//	uhttps_log_error("[warn] ignored HTTP request header \"%s: %s\"", header_line_field, header_line_values);
		// else ignore the header
		// TODO? per le richieste PUT è necessario annullare il parsing se è presente anche solo un header non supportato
	
		// skip to the next line
		while(*in_request && *in_request!='\n') in_request++;
		 in_request++;
		
		// reset current field and value string
		*header_line_field='\0';
		*header_line_values='\0';
	} //endwhile

	return;
}


void uhttps_parse_http_response_headers( char* out_response, struct uttps_http_response_parsed* in_response ) // TODO: controllare dopo ogni sprintf che l'offset non superi 4096 bytes...
{
	if( !in_response || !out_response );
		
	// scrive la Initial Response Line (Status Line)
	size_t out_response_offset =
		sprintf(out_response, "HTTP/%s %d %s\r\n", in_response->http_version, in_response->status_code, in_response->status_code_description );
	
	// se non sono vuoti, scrive i vari headers
	if( *(in_response->location) != '\0' )
	{
		out_response_offset +=
			sprintf(out_response+out_response_offset, "Location: %s\r\n", in_response->location );
	}
	if( *(in_response->server) != '\0' )
	{
		out_response_offset +=
			sprintf(out_response+out_response_offset, "Server: %s\r\n", in_response->server );
	}
	if( *(in_response->date) != '\0' )
	{
		out_response_offset +=
			sprintf(out_response+out_response_offset, "Date: %s\r\n", in_response->date );
	}
	if( *(in_response->content_type) != '\0' )
	{
		out_response_offset +=
			sprintf(out_response+out_response_offset, "Content-Type: %s\r\n", in_response->content_type );
	}
	if( in_response->content_length != 0 )
	{
		out_response_offset +=
			sprintf(out_response+out_response_offset, "Content-Length: %d\r\n", in_response->content_length );
	}
	if( *(in_response->www_authenticate) != 0 )
	{
		out_response_offset +=
			sprintf(out_response+out_response_offset, "WWW-Authenticate: %s\r\n", in_response->www_authenticate );
	}
	// TODO: ALTRI HEADERS...

	// inserisce un newline come marcatore d'inizio del body
	// OSS.: alcuni browser interpretano la response come incompleta/malformed se questo newline non è presente
	out_response_offset +=
		sprintf(out_response+out_response_offset, "\r\n");
	
// DEBUG: puts("response:");
// DEBUG: puts(out_response);

	in_response->_headers_size = out_response_offset;
	
	return;
}


