#include "uhttps.h"
#include "cfg_parser.h"
#include "logger.h"
#include <time.h>
#include <stdarg.h>
#include <stdio.h>

static FILE* LOGFILE = NULL;
static FILE* ERRFILE = NULL;

static void _get_current_date( char* s, size_t size )
{
	time_t now;
	time(&now);
	
	#if defined(WINDOWS) || defined(WIN32) || defined(_WIN32)
		struct tm* d = localtime(&now); // on Windows "localtime" is thread-safe... -> http://old.nabble.com/Porting-localtime_r-and-gmtime_r-td15282276.html
	#else
		struct tm date;
		struct tm* d = localtime_r(&now, &date); // ...on POSIX it is not.
	#endif
	
	strftime(s, size, "%d/%b/%Y:%H:%M:%S %Z", d);
}


void uhttps_logfile_open(const String log_pathname)
{
	if( strcmp(log_pathname, "stdout") == 0 )
	{
		uhttps_log_error("[notice] using stdout as log file");
		LOGFILE = stdout;
		return;
	}
	if( strcmp(log_pathname, "stderr") == 0 )
	{
		uhttps_log_error("[notice] using stderr as log file");
		LOGFILE = stderr;
		return;
	}
	// altrimenti bisogna aprire il file in appending + text mode
	LOGFILE = fopen( log_pathname, "a" );
	if( !LOGFILE )
	{
		uhttps_log_error( "[error] unable to open log file \"%s\" for appending", log_pathname );
		LOGFILE = stdout;
	}
}

void uhttps_logfile_close()
{
	if( LOGFILE == stdout ) return; // no need to close stdout
	if( LOGFILE == stderr ) return; // no need to close stderr
	//else
	fclose(LOGFILE);
}


void uhttps_log_request(String host, String user, String request, int status, int bytes)
{
	if( !LOGFILE ) LOGFILE = stdout;
	
	// OSS: poichè è effettuata una sola operazione di I/O non c'è il rischio di interferenze con gli altri threads
	
	char current_date[256] = {0};
	_get_current_date(current_date, 256);
	fprintf( LOGFILE, "%s - %s [%s] \"%s\" %d %d\n", host, user, current_date, request, status, bytes);
	
	fflush( LOGFILE ); // necessario solo su Windows?
}

void uhttps_log_error( char* fmt, ... )
{
	if( !ERRFILE ) ERRFILE = stderr;
	
	// OSS: poichè sono effettuate + operazioni di I/O è necessario serializzarle:
		#if defined(WINDOWS) || defined(WIN32) || defined(_WIN32)
			// WANTED: primitiva di serializzazione -> usare CRITICAL_SECTION?
		#else
			flockfile( ERRFILE );
		#endif
	
	char current_date[256] = {0};
	_get_current_date(current_date, 256);
	fprintf( ERRFILE, "[%s] ", current_date);

	va_list args;
	va_start( args, fmt );
		vfprintf( ERRFILE, fmt, args );
	va_end( args );
	
	fputs("\n", ERRFILE);
	fflush( ERRFILE );
	
		#if defined(WINDOWS) || defined(WIN32) || defined(_WIN32)
			// unlock a CRITICAL_SECTION?
		#else
			funlockfile( logfile );
		#endif
}
