#include "uhttps.h"
#include "usersdb.h"


void uhttps_usersdb_read( struct users_db* out_usersdb, const char* usersdb_pathname )
{
	if(!out_usersdb) return;
		
	// apre il file DB degli utenti in sola lettura
	FILE* usersdb_file = fopen(usersdb_pathname, "r");
	if( !usersdb_file )
	{
		uhttps_log_error( "[fatal error] no users database file \"%s\" found", usersdb_pathname );
		exit(2);
	}

	// conta il numero di utenti registrati nel DB (senza parsarli)
	const int MAX_USERS_DB_LINE_LENGHT = _UHTTPS_MAX_USERNAME_LENGHT + _UHTTPS_MAX_PASSWORD_LENGHT + 5;
	char current_line[ MAX_USERS_DB_LINE_LENGHT ]; memset(current_line,0,sizeof(current_line));
	int users_count = 0;
	while( fgets( current_line, MAX_USERS_DB_LINE_LENGHT, usersdb_file ) != NULL )
	{
		// se la linea è vuota o inizia con "#" (commento), la ignora
		if(*current_line=='\r' || *current_line=='\n' || *current_line=='#') continue;
		// altrimenti la considera una linea valida -> se non è valida potrebbe causare un lieve spreco di spazio allocato...
		users_count++;
	}
	
	// alloca il DB di utenti in memoria
	out_usersdb->data = calloc( users_count, sizeof(struct user) );
	if( (out_usersdb->data) == NULL )
	{
		uhttps_log_error( "[fatal error] unable to allocate memory for the users database", usersdb_pathname );
		exit(2);
	}
	
	// resetta la posizione del file all'inizio
	rewind( usersdb_file );
	
	// legge e parsa una riga alla volta
	char current_username[ _UHTTPS_MAX_USERNAME_LENGHT ] = {0};
	char current_password[ _UHTTPS_MAX_PASSWORD_LENGHT ] = {0};
	int current_line_count = 0;
	int out_usersdb_current_position = 0; // indice per l'array
	while( fgets( current_line, MAX_USERS_DB_LINE_LENGHT, usersdb_file ) != NULL )
	{
		current_line_count++;
		
		// se la linea è vuota o inizia con "#" (commento), la ignora
		if(*current_line=='\r' || *current_line=='\n' || *current_line=='#') continue;
		// altrimenti prova il parsing
		*current_username='\0'; *current_password='\0'; // svuota le stringhe
		sscanf( current_line, "%[^:]:%s", current_username, current_password );
		if(!*current_username || !*current_password) // se una delle due stringhe è ancora vuota il parsing è fallito
		{
			uhttps_log_error("[error] invalid line %d in users DB file, skipping", current_line_count);
			users_count--;
			continue;
		}
		// else
		strcpy( (out_usersdb->data[ out_usersdb_current_position ]).username, current_username);
		strcpy( (out_usersdb->data[ out_usersdb_current_position ]).password, current_password);
		out_usersdb_current_position++;
	}
	
	fclose( usersdb_file );
	
	if( users_count == 0 )
	{
		uhttps_log_error("[fatal error] no valid user account found in \"%s\"", usersdb_pathname);
		exit(1);
	}
	// else
	out_usersdb->nelem = users_count;
	uhttps_log_error( "[notice] %d user accounts read from \"%s\"", users_count, usersdb_pathname );

}

void uhttps_usersdb_free( struct users_db* out_usersdb )
{
	free(out_usersdb->data);
	out_usersdb->data = NULL;
	out_usersdb->nelem = 0;
}

boolean uhttps_usersdb_login( const struct users_db* usersdb, const char* username, const char* password )
{
	int i;
	for( i=0; i<(usersdb->nelem); i++ )
	{
		if(*((usersdb->data[i]).username)=='\0') break;
		if( strcmp((usersdb->data[i]).username, username) == 0 )
			if( strcmp((usersdb->data[i]).password, password) == 0 )
				return true;
	}
	return false;
}
